@@ -15,7 +15,7 @@
 
 using namespace Qt::Literals::StringLiterals;
 
-AbstractAccount::AbstractAccount(QObject *parent, const QString &instanceUri)
+AbstractAccount::AbstractAccount(const QString &instanceUri, QObject *parent)
     : QObject(parent)
     , m_instance_uri(instanceUri)
     // default to 500, instances which support more signal it
@@ -24,28 +24,23 @@ AbstractAccount::AbstractAccount(QObject *parent, const QString &instanceUri)
     , m_supportsLocalVisibility(false)
     , m_charactersReservedPerUrl(23)
     , m_identity(std::make_shared<Identity>())
-    , m_allowedContentTypes(AllowedContentType::PlainText)
-{
-}
-
-AbstractAccount::AbstractAccount(QObject *parent)
-    : QObject(parent)
-    // default to 500, instances which support more signal it
-    , m_maxPostLength(500)
-    , m_maxPollOptions(4)
-    , m_supportsLocalVisibility(false)
-    , m_charactersReservedPerUrl(23)
-    , m_identity(std::make_shared<Identity>())
-    , m_allowedContentTypes(AllowedContentType::PlainText)
+    , m_preferences(new Preferences(this))
 {
+    // Test code uses a blank instance URI
+    if (!AccountManager::instance().testMode()) {
+        Q_ASSERT(!instanceUri.isEmpty());
+    }
 }
 
 AccountConfig *AbstractAccount::config()
 {
     if (!m_config) {
-        m_config = new AccountConfig{settingsGroupName(), this};
+        // do not write to settings if we do not have complete information yet,
+        // or else it writes malformed and possibly duplicate accounts to settings.
+        if (!m_name.isEmpty() && !m_instance_uri.isEmpty()) {
+            m_config = new AccountConfig{settingsGroupName(), this};
+        }
     }
-    Q_ASSERT(m_config != nullptr);
     return m_config;
 }
 
@@ -103,11 +98,6 @@ bool AbstractAccount::hasName() const
     return !m_name.isEmpty();
 }
 
-bool AbstractAccount::hasInstanceUrl() const
-{
-    return !m_instance_uri.isEmpty();
-}
-
 QUrl AbstractAccount::apiUrl(const QString &path) const
 {
     QUrl url = QUrl::fromUserInput(m_instance_uri);
@@ -120,7 +110,7 @@ QUrl AbstractAccount::apiUrl(const QString &path) const
 void AbstractAccount::registerApplication(const QString &appName, const QString &website, const QString &additionalScopes, const bool useAuthCode)
 {
     // clear any previous bearer token credentials
-    m_token = QString();
+    m_token.clear();
 
     QString ourAdditionalScopes;
 #ifdef HAVE_KUNIFIEDPUSH
@@ -144,22 +134,33 @@ void AbstractAccount::registerApplication(const QString &appName, const QString
 
     fetchInstanceMetadata();
 
-    post(regUrl, doc, false, this, [=](QNetworkReply *reply) {
-        if (!reply->isFinished()) {
-            qCDebug(TOKODON_LOG) << "not finished";
-            return;
-        }
+#ifdef HAVE_KUNIFIEDPUSH
+    connect(
+        this,
+        &AbstractAccount::authenticated,
+        this,
+        [this] {
+            if (successfullyAuthenticated()) {
+                // We asked for the push scope, so we can safely start subscribing to notifications
+                config()->setEnablePushNotifications(true);
+                config()->save();
+            }
+        },
+        Qt::SingleShotConnection);
+#endif
 
+    post(regUrl, doc, false, this, [=](QNetworkReply *reply) {
         const auto data = reply->readAll();
         const auto doc = QJsonDocument::fromJson(data);
 
         m_client_id = doc.object()["client_id"_L1].toString();
         m_client_secret = doc.object()["client_secret"_L1].toString();
 
-        s_messageFilter->insert(m_client_secret, QStringLiteral("CLIENT_SECRET"));
-
-        if (isRegistered()) {
+        if (!m_client_id.isEmpty() && !m_client_secret.isEmpty()) {
+            QMessageFilterContainer::self()->insert(m_client_secret, QStringLiteral("CLIENT_SECRET"));
             Q_EMIT registered();
+        } else {
+            // TODO: emit an error here
         }
     });
 }
@@ -172,9 +173,10 @@ void AbstractAccount::registerAccount(const QString &username,
                                       const QString &reason)
 {
     // get an app-level access token, obviously we don't have a user token yet.
-    const QUrl tokenUrl = getTokenUrl();
-    QUrlQuery q = buildOAuthQuery();
+    const QUrl tokenUrl = apiUrl(QStringLiteral("/oauth/token"));
+    QUrlQuery q;
 
+    q.addQueryItem(QStringLiteral("client_id"), m_client_id);
     q.addQueryItem(QStringLiteral("client_secret"), m_client_secret);
     q.addQueryItem(QStringLiteral("grant_type"), QStringLiteral("client_credentials"));
     q.addQueryItem(QStringLiteral("scope"), QStringLiteral("write"));
@@ -183,9 +185,8 @@ void AbstractAccount::registerAccount(const QString &username,
         auto data = reply->readAll();
         auto doc = QJsonDocument::fromJson(data);
 
-        // override the token for now
+        // We do not use setAccessToken here because it's a temporary one, and cannot be validated.
         m_token = doc.object()["access_token"_L1].toString();
-        s_messageFilter->insert(m_token, QStringLiteral("ACCESS_TOKEN"));
 
         const QUrlQuery formdata{
             {QStringLiteral("username"), username},
@@ -295,16 +296,12 @@ bool AbstractAccount::identityCached(const QString &accountId) const
     return id && id->id() == accountId;
 }
 
-QUrlQuery AbstractAccount::buildOAuthQuery() const
-{
-    return QUrlQuery{{QStringLiteral("client_id"), m_client_id}};
-}
-
 QUrl AbstractAccount::getAuthorizeUrl() const
 {
     QUrl url = apiUrl(QStringLiteral("/oauth/authorize"));
-    QUrlQuery q = buildOAuthQuery();
+    QUrlQuery q;
 
+    q.addQueryItem(QStringLiteral("client_id"), m_client_id);
     q.addQueryItem(QStringLiteral("redirect_uri"), m_redirectUri);
     q.addQueryItem(QStringLiteral("response_type"), QStringLiteral("code"));
     q.addQueryItem(QStringLiteral("scope"), QStringLiteral("read write follow ") + m_additionalScopes);
@@ -317,16 +314,8 @@ QUrl AbstractAccount::getAuthorizeUrl() const
 void AbstractAccount::setAccessToken(const QString &token)
 {
     m_token = token;
-    s_messageFilter->insert(m_token, QStringLiteral("ACCESS_TOKEN"));
-    AccountManager::instance().addAccount(this, false);
-    // Future programmer: make sure not to set explicitUserAction here because it needs the settingsGroupName() to be valid first
-    AccountManager::instance().selectAccount(this, false);
-    validateToken(true);
-}
-
-QUrl AbstractAccount::getTokenUrl() const
-{
-    return apiUrl(QStringLiteral("/oauth/token"));
+    QMessageFilterContainer::self()->insert(m_token, QStringLiteral("ACCESS_TOKEN"));
+    validateToken();
 }
 
 void AbstractAccount::setInstanceUri(const QString &instance_uri)
@@ -345,9 +334,10 @@ QString AbstractAccount::instanceUri() const
 
 void AbstractAccount::setToken(const QString &authcode)
 {
-    const QUrl tokenUrl = getTokenUrl();
-    QUrlQuery q = buildOAuthQuery();
+    const QUrl tokenUrl = apiUrl(QStringLiteral("/oauth/token"));
+    QUrlQuery q;
 
+    q.addQueryItem(QStringLiteral("client_id"), m_client_id);
     q.addQueryItem(QStringLiteral("client_secret"), m_client_secret);
     q.addQueryItem(QStringLiteral("redirect_uri"), m_redirectUri);
     q.addQueryItem(QStringLiteral("grant_type"), QStringLiteral("authorization_code"));
@@ -422,53 +412,6 @@ void AbstractAccount::unpin(Post *p)
     mutatePost(p->postId(), QStringLiteral("unpin"));
 }
 
-// It seemed clearer to keep this logic separate from the general instance metadata collection, on the off chance
-// that it might need to be extended later on.
-static AbstractAccount::AllowedContentType parseVersion(const QString &instanceVer)
-{
-    using ContentType = AbstractAccount::AllowedContentType;
-
-    unsigned int result = ContentType::PlainText;
-    if (instanceVer.contains("glitch"_L1)) {
-        result |= ContentType::Markdown | ContentType::Html;
-    }
-
-    return static_cast<ContentType>(result);
-}
-
-static QMap<QString, AbstractAccount::AllowedContentType> stringToContentType = {
-    {QStringLiteral("text/plain"), AbstractAccount::AllowedContentType::PlainText},
-    {QStringLiteral("text/bbcode"), AbstractAccount::AllowedContentType::BBCode},
-    {QStringLiteral("text/html"), AbstractAccount::AllowedContentType::Html},
-    {QStringLiteral("text/markdown"), AbstractAccount::AllowedContentType::Markdown},
-};
-
-static AbstractAccount::AllowedContentType parsePleromaInfo(const QJsonDocument &doc)
-{
-    using ContentType = AbstractAccount::AllowedContentType;
-    unsigned int result = ContentType::PlainText;
-
-    auto obj = doc.object();
-    if (obj.contains("metadata"_L1)) {
-        auto metadata = obj["metadata"_L1].toObject();
-        if (!metadata.contains("postFormats"_L1))
-            return static_cast<ContentType>(result);
-
-        auto formats = metadata["postFormats"_L1].toArray();
-
-        for (auto c : formats) {
-            auto fmt = c.toString();
-
-            if (!stringToContentType.contains(fmt))
-                continue;
-
-            result |= (unsigned int)stringToContentType[fmt];
-        }
-    }
-
-    return static_cast<ContentType>(result);
-}
-
 void AbstractAccount::fetchInstanceMetadata()
 {
     get(
@@ -476,14 +419,12 @@ void AbstractAccount::fetchInstanceMetadata()
         false,
         this,
         [=](QNetworkReply *reply) {
-            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
-                return;
-
             const auto data = reply->readAll();
             const auto doc = QJsonDocument::fromJson(data);
 
-            if (!doc.isObject())
+            if (!doc.isObject()) {
                 return;
+            }
 
             const auto obj = doc.object();
 
@@ -497,11 +438,6 @@ void AbstractAccount::fetchInstanceMetadata()
                 }
             }
 
-            // One can only hope that there will always be a version attached
-            if (obj.contains("version"_L1)) {
-                m_allowedContentTypes = parseVersion(obj["version"_L1].toString());
-            }
-
             // Pleroma/Akkoma may report maximum post characters here, instead
             if (obj.contains("max_toot_chars"_L1)) {
                 m_maxPostLength = obj["max_toot_chars"_L1].toInt();
@@ -532,14 +468,12 @@ void AbstractAccount::fetchInstanceMetadata()
             // Fall back to v1 instance information
             // TODO: a lot of this can be merged with v2 handling
             get(apiUrl(QStringLiteral("/api/v1/instance")), false, this, [=](QNetworkReply *reply) {
-                if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
-                    return;
-
                 const auto data = reply->readAll();
                 const auto doc = QJsonDocument::fromJson(data);
 
-                if (!doc.isObject())
+                if (!doc.isObject()) {
                     return;
+                }
 
                 const auto obj = doc.object();
 
@@ -553,11 +487,6 @@ void AbstractAccount::fetchInstanceMetadata()
                     }
                 }
 
-                // One can only hope that there will always be a version attached
-                if (obj.contains("version"_L1)) {
-                    m_allowedContentTypes = parseVersion(obj["version"_L1].toString());
-                }
-
                 // Pleroma/Akkoma may report maximum post characters here, instead
                 if (obj.contains("max_toot_chars"_L1)) {
                     m_maxPostLength = obj["max_toot_chars"_L1].toInt();
@@ -583,22 +512,9 @@ void AbstractAccount::fetchInstanceMetadata()
             });
         });
 
-    get(apiUrl(QStringLiteral("/nodeinfo/2.1.json")), false, this, [=](QNetworkReply *reply) {
-        const auto data = reply->readAll();
-        const auto doc = QJsonDocument::fromJson(data);
-
-        m_allowedContentTypes = parsePleromaInfo(doc);
-        Q_EMIT fetchedInstanceMetadata();
-    });
-
     fetchCustomEmojis();
 }
 
-void AbstractAccount::invalidate()
-{
-    Q_EMIT invalidated();
-}
-
 void AbstractAccount::saveTimelinePosition(const QString &timeline, const QString &lastReadId)
 {
     QUrl uri = apiUrl(QStringLiteral("/api/v1/markers"));
@@ -610,11 +526,6 @@ void AbstractAccount::saveTimelinePosition(const QString &timeline, const QStrin
     post(uri, urlQuery, true, this, {});
 }
 
-void AbstractAccount::invalidatePost(Post *p)
-{
-    Q_EMIT invalidatedPost(p);
-}
-
 QUrl AbstractAccount::streamingUrl(const QString &stream)
 {
     QUrl url = apiUrl(QStringLiteral("/api/v1/streaming"));
@@ -813,11 +724,6 @@ void AbstractAccount::fetchOEmbed(const QString &id, Identity *identity)
     });
 }
 
-bool AbstractAccount::isRegistered() const
-{
-    return !m_client_id.isEmpty() && !m_client_secret.isEmpty();
-}
-
 bool AbstractAccount::registrationsOpen() const
 {
     return m_registrationsOpen;
@@ -847,7 +753,7 @@ void AbstractAccount::fetchCustomEmojis()
 {
     m_customEmojis.clear();
 
-    get(apiUrl(QStringLiteral("/api/v1/custom_emojis")), false, this, [=](QNetworkReply *reply) {
+    get(apiUrl(QStringLiteral("/api/v1/custom_emojis")), true, this, [=](QNetworkReply *reply) {
         if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
             return;
 
