@@ -3,6 +3,7 @@
 
 #include "utils/ecdh.h"
 
+#include <openssl/ec.h>
 #include <openssl/obj_mac.h>
 
 constexpr int PRIVATE_KEY_LENGTH = 32;
@@ -10,6 +11,7 @@ constexpr int PUBLIC_KEY_LENGTH = 65;
 
 EC_KEY *generateECDHKeypair()
 {
+    // TODO: port all of these functions from the legacy API
     EC_KEY *key = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
     if (!key) {
         return nullptr;
